#ifndef DEFORMOBJ_H
#define DEFORMOBJ_H

#include <cstring>
#include "vec3i.h"
#include "vec2f.h"
#include "element.h"
#include "externalforces.h"

struct DeformObj{
   int n; // number of nodes
   int nt; // number of triangles
   std::vector<Vec3i> tri;
   std::vector<Element> stiffness;
   std::vector<Vec2f> restx, x, v, f, oldx;
   std::vector<float> mass;
   std::vector<int> surface_nodes;
   float youngs_modulus, poisson_ratio, damping, density;
   float min_mesh_length;
   float maxdt;
   ExternalForces external_forces;

   DeformObj()
      : n(0), nt(0), youngs_modulus(0), poisson_ratio(0), damping(0), density(0), min_mesh_length(0), maxdt(0)
   {}

   void initialize_block_mesh(const Vec2f &centre, float length, float mesh_size);
   void compute_mass(float density);
   void compute_stiffness(float youngs_modulus, float poisson_ratio);
   void set_damping(float damping);
   void advance(float dt);
   void compute_forces(void);
   void add_triangle_forces(int t);
   void compute_max_timestep(void);

   void save_positions(void)
   {
      for(int i=0; i<n; ++i)
	 oldx[i]=x[i];
   }
};

#endif
